--       Redistribution and use in source and binary forms, with or without
--       modification, are permitted provided that the following conditions are
--       met:
--       
--       * Redistributions of source code must retain the above copyright
--         notice, this list of conditions and the following disclaimer.
--       * Redistributions in binary form must reproduce the above
--         copyright notice, this list of conditions and the following disclaimer
--         in the documentation and/or other materials provided with the
--         distribution.
--       * Neither the name of the  nor the names of its
--         contributors may be used to endorse or promote products derived from
--         this software without specific prior written permission.
--       
--       THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
--       "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
--       LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
--       A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
--       OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
--       SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
--       LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
--       DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
--       THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
--       (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
--       OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--
--	Copyright (C) 2008,2009 Aleksandr Deviatkin aka alid


--  
create table mgroup (
	id integer not null,
	name varchar,
	memo varchar
);

-- 
create table counters (
	id integer not null,
	name varchar,
	addr integer,
	mgroup integer,
	passwd varchar,
	passwd2 varchar,
	sn integer,
	model integer,
	ktrans integer default 1,	--  
	setdate timestamp,	--  
	memo varchar,
	active integer,
	modtime timestamp default now()
);


--  
create table counter_type (
	id integer not null,
	name varchar,
	iface integer
);

-- 
create table iface (
	id integer not null,
	name varchar,
	dev varchar
);

-- 
create table monitor (
	date integer not null,
	counter integer not null,
	mv1 decimal,	--   1
	mv2 decimal,	--   2
	mv3 decimal,	--   3
	mc1 decimal,	--   1
	mc2 decimal,	--   2
	mc3 decimal,	--   3
	mf decimal,		-- 
	ma1 decimal,	--    1-2
	ma2 decimal,	--    1-3
	ma3 decimal,	--    2-3
	mps decimal,	--  P 
	mp1 decimal,	--  P 1
	mp2 decimal,	--  P 2
	mp3 decimal,	--  P 3
	mqs decimal,	--  Q 
	mq1 decimal,	--  Q 1
	mq2 decimal,	--  Q 2
	mq3 decimal,	--  Q 3
	mss decimal,	--  S 
	ms1 decimal,	--  S 1
	ms2 decimal,	--  S 2
	ms3 decimal,	--  S 3
	mks decimal,	-- . 
	mk1 decimal,	-- . 1
	mk2 decimal,	-- . 2
	mk3 decimal,	-- . 3
	se1ai decimal, --   1 A-import
	se1ae decimal, --   1 A-export
	se1ri decimal, --   1 R-import
	se1re decimal, --   1 R-export
	se2ai decimal, --   2 A-import
	se2ae decimal, --   2 A-export
	se2ri decimal, --   2 R-import
	se2re decimal, --   2 R-export
	ise decimal		--  
);
CREATE INDEX monitor_date_i ON monitor (date);
CREATE INDEX monitor_counter_i ON monitor (counter);
create VIEW m_monitor as SELECT to_timestamp(monitor."date"::double precision) as tm,* from monitor;
CREATE VIEW m_power AS select to_char(to_timestamp(monitor.date::double precision),'DD.MM.YYYY HH24:MI:SS') AS date,to_char(to_timestamp(monitor.date::double precision),'DD.MM.YYYY') AS dt,to_char(to_timestamp(monitor.date::double precision),'HH24:MI:SS') AS tm,monitor.counter AS counter,to_char(monitor.mp1,'999999') AS p1,to_char(monitor.mp2,'999999') AS p2,to_char(monitor.mp3,'999999') AS p3,to_char(monitor.mps,'999999') AS p,to_char(monitor.ise*1000,'999999') AS pi from monitor;

create view m_stored_energy as select to_char(to_timestamp(monitor.date::double precision), 'DD.MM.YYYY HH24:MI:SS'::text) AS datime, to_char(to_timestamp(monitor.date::double precision), 'DD.MM.YYYY'::text) AS dt, to_char(to_timestamp(monitor.date::double precision), 'HH24:MI:SS'::text) AS tm,date,counter,counters.addr,counters.name,counters.memo,se1ai,se2ai from monitor inner join counters on counter=counters.id order by date desc;

CREATE FUNCTION ise_stamp() RETURNS trigger AS '
DECLARE
   prev record;
BEGIN
   SELECT INTO prev counters.ktrans,date,se1ai,se2ai from monitor inner join counters on counter=counters.id where date<NEW.date and counter=NEW.counter order by date desc limit 1;
   IF NOT FOUND THEN
       NEW.ise := 0;
   ELSE
       NEW.ise := ((NEW.se1ai + NEW.se2ai) - (prev.se1ai + prev.se2ai)) * 3600 * prev.ktrans /(NEW.date - prev.date);
   END IF;
   RETURN NEW;
END;
' LANGUAGE plpgsql;

CREATE TRIGGER ise_stamp BEFORE INSERT ON monitor FOR EACH ROW EXECUTE PROCEDURE ise_stamp();
