#!/usr/bin/perl -w
use strict;
$| = 1;

#       Redistribution and use in source and binary forms, with or without
#       modification, are permitted provided that the following conditions are
#       met:
#       
#       * Redistributions of source code must retain the above copyright
#         notice, this list of conditions and the following disclaimer.
#       * Redistributions in binary form must reproduce the above
#         copyright notice, this list of conditions and the following disclaimer
#         in the documentation and/or other materials provided with the
#         distribution.
#       * Neither the name of the  nor the names of its
#         contributors may be used to endorse or promote products derived from
#         this software without specific prior written permission.
#       
#       THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#       "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#       LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#       A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#       OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#       SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#       LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#       DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#       THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#       (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#       OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#	Copyright (C) 2008,2009 Aleksandr Deviatkin aka alid

#
#      -203.2
# 2009-03-22 alid
#

use IO::File;
use Device::SerialPort;

my (@args, %opts);
foreach(@ARGV){
   if(/^\-(\S)(.*)/){	$opts{$1} = $2;	} else {	push @args, $_;	}
}
my $verb = exists $opts{v};
my $retries = (exists $opts{r}) ? $opts{r} : 10;
my $showhead = exists $opts{s};	#     
my ($saddr, $device) = (@args);
die "Usage: $0 addr serial-dev"	unless(defined $saddr && $device);

my $addr = sprintf("%08x",$saddr);
$addr =~ s/(\w\w)(\w\w)(\w\w)(\w\w)/$1 $2 $3 $4/;
print "Addr: [$addr]\n"	if $verb;

my $mbtx = "/var/www/counter/mbtx";
my $crcu = "/var/www/counter/crc";
my $STALL_DEFAULT=2;	# how many seconds to wait for new input
my $MAXLENGTH = 255;	# #    
my $port=Device::SerialPort->new("$device");
my ($status,$cnt,@data);

if($verb) {
	print "Connection testing ... "	if $verb;
	$status = tst($device,$port,$addr);
	print "$status\n"	if $verb;
	die	"[$addr] Connection failed: [$status]"	unless($status=~/ok/);
}

my $Data;
#
#  
#
print "Stored energy ########################\n"	if $verb;
($status,$cnt,@data) = get($device,$addr,'27');
die "[$addr] 27h request failed: [$status]"	unless($status=~/ok/);
print "[$status][$cnt][".join(' ',@data)."]\n"	if $verb;
my @a;
foreach my $i (5,9) {
	my $num = join("",$data[0+$i],$data[1+$i],$data[2+$i],$data[3+$i]);
	push @{$Data->{se}}, $num/100;
}
print "se [27h]: ".join(' ',@{$Data->{se}})."\n"	if $verb;


#
#   
#
print "Monitoring ########################\n"	if $verb;
#  
($status,$cnt,@data) = get($device,$addr,'26');
#die "[$addr] 26h request failed: [$status]"	unless($status=~/ok/);
print "[$status][$cnt][".join(' ',@data)."]\n"	if $verb;
push @{$Data->{mp}}, join("",$data[5],$data[6])/100;
print "mp [26h]: ".join(' ',@{$Data->{mp}})."\n"	if $verb;

# 
($status,$cnt,@data) = get($device,$addr,'81');
die "[$addr] 81h request failed: [$status]"	unless($status=~/ok/);
print "[$status][$cnt][".join(' ',@data)."]\n"	if $verb;
push @{$Data->{mf}}, join("",$data[5],$data[6])/100;
print "mf [81h]: ".join(' ',@{$Data->{mf}})."\n"	if $verb;

# # , ,  (1 ) 
($status,$cnt,@data) = get($device,$addr,'63');
die "[$addr] 63h request failed: [$status]"	unless($status=~/ok/);
print "[$status][$cnt][".join(' ',@data)."]\n"	if $verb;
push @{$Data->{mv}}, join("",$data[5],$data[6])/10;
print "mv [63h]: ".join(' ',@{$Data->{mv}})."\n"	if $verb;
push @{$Data->{mc}}, join("",$data[7],$data[8])/10;
print "mv [63h]: ".join(' ',@{$Data->{mc}})."\n"	if $verb;
push @{$Data->{ms}}, join("",$data[9],$data[10],$data[11])/10;
print "ms [63h]: ".join(' ',@{$Data->{ms}})."\n"	if $verb;

###### data output
print "addr;mv1;mc1;mf;mps;mss;se1ai;se2ai\n"	if($showhead);
print "$saddr;".$Data->{mv}[0].";".
	$Data->{mc}[0].";".
	$Data->{mf}[0].";".
	$Data->{mp}[0].";".		#  P
	$Data->{ms}[0].";".		#  S
	join(";",@{$Data->{se}}).
	"\n";

###################### subs
#   
sub tst {
	my ($device,$port,$addr) = @_;
	my $cmd = '2f';
	my $i = $retries;
	my $res;
	do {
		_send($device,$addr,$cmd);
		$res = isok($device,$port,$addr);
		return $res	if($res =~ /ok/);
		$i--;
	} while($i);
	return $res;
}

##########################################################################
sub isok {	#       
	my ($device,$port,$addr) = @_;
	my ($status,$cnt,@data) = _recv($port);
	if($status =~ /ok/) {
		unless($cnt eq '11' && $data[4] eq '2f') {
			$status = 'fail';
		}
	}
	return $status;
}

#  crc 
sub iscrc {
	my (@data) = @_;
	my $icrc = pop(@data).pop(@data);
	my $str = join(" ",@data);
	if(`echo '$str' | $crcu` =~ /^\[(\w\w\w\w)\]\s/) {
		return 'ok'	if(uc($1) eq uc($icrc));
	}
	return 'crc-error';
}

sub _send {
	my ($device,$addr, @str) = @_;
	open(SO, "| $mbtx -s $device -q");
	print SO "$addr ".join(' ',@str)."\n";
	close SO;
}

sub _recv {
	my ($port) = @_;
	my $timeout=$STALL_DEFAULT * 10;
	$port->read_char_time(0);     # don't wait for each character
	$port->read_const_time(200); # 0,15 second per unfulfilled "read" call
	my $status='ok';
	my $chars=0;
	my @data;
	my $buffer="";
	my ($count,$saw);
	while ($timeout>0) {
		($count,$saw)=$port->read($MAXLENGTH); # will read _up to_ $MAXLENGTH chars
		if ($count > 0) {
			$chars+=$count;
			$buffer.=$saw;
			@data = map {/(..)/gm} unpack("H*",$buffer);
			last;
		}
		else {
			$timeout--;
		}
	}
	if ($timeout==0) {
		$status = 0;	# Waited $STALL_DEFAULT seconds and never saw what I wanted
	}
	$status = iscrc(@data)	if(@data);
	return($status,$count,@data);
}

sub get {
	my ($device,$addr,$ts) = @_;
	my $i = $retries;
	do {
		_send($device,$addr,$ts);
		($status,$cnt,@data) = _recv($port);
		$i--;
		$i=0	if($status =~ /ok/);
	} while($i);
	return($status,$cnt,@data);
}







